# Generated by Django 2.1.1 on 2018-09-08 18:42

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('django_fsm_log', '0003_statelog_description'),
    ]

    operations = [
        migrations.AddField(
            model_name='statelog',
            name='source_state',
            field=models.CharField(blank=True, db_index=True, default=None, max_length=255, null=True),
        ),
        migrations.AlterField(
            model_name='statelog',
            name='state',
            field=models.CharField(db_index=True, max_length=255, verbose_name='Target state'),
        ),
    ]



from __future__ import unicode_literals
from django.apps import AppConfig
from django.conf import settings
from django.utils.module_loading import import_string

from django_fsm.signals import pre_transition, post_transition


class DjangoFSMLogAppConfig(AppConfig):
    name = 'django_fsm_log'
    verbose_name = "Django FSM Log"

    def ready(self):
        backend = import_string(settings.DJANGO_FSM_LOG_STORAGE_METHOD)
        StateLog = self.get_model('StateLog')

        backend.setup_model(StateLog)

        pre_transition.connect(backend.pre_transition_callback)
        post_transition.connect(backend.post_transition_callback)



from django.contrib.auth import get_user_model
from django_fsm_log.managers import PendingStateLogManager
from django_fsm_log.models import StateLog
import pytest

from .models import Article, ArticleInteger


@pytest.fixture
def article(db, request, settings):
    if 'ignore_article' in request.keywords:
        settings.DJANGO_FSM_LOG_IGNORED_MODELS = ['tests.models.Article']
    return Article.objects.create(state='draft')


@pytest.fixture
def article_integer(db, request, settings):
    return ArticleInteger.objects.create()


@pytest.fixture
def article2(db):
    return Article.objects.create(state='draft')


@pytest.fixture
def user(db):
    User = get_user_model()
    return User.objects.create_user(username='jacob', password='password')


@pytest.fixture(autouse=True)
def pending_objects(db, request):
    if 'pending_objects' in request.keywords:
        if not hasattr(StateLog, 'pending_objects'):
            StateLog.add_to_class('pending_objects', PendingStateLogManager())



NOTSET = object()


class FSMLogDescriptor(object):

    ATTR_PREFIX = '__django_fsm_log_attr_'

    def __init__(self, instance, attribute, value=NOTSET):
        self.instance = instance
        self.attribute = attribute
        if value is not NOTSET:
            self.set(value)

    def get(self):
        return getattr(self.instance, self.ATTR_PREFIX + self.attribute)

    def set(self, value):
        setattr(self.instance, self.ATTR_PREFIX + self.attribute, value)

    def __enter__(self):
        return self

    def __exit__(self, *args):
        try:
            delattr(self.instance, self.ATTR_PREFIX + self.attribute)
        except AttributeError:
            pass



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
import django.utils.timezone
from django.conf import settings


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('contenttypes', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='StateLog',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('timestamp', models.DateTimeField(default=django.utils.timezone.now)),
                ('state', models.CharField(max_length=255, db_index=True)),
                ('transition', models.CharField(max_length=255)),
                ('object_id', models.PositiveIntegerField(db_index=True)),
                ('by', models.ForeignKey(blank=True, to=settings.AUTH_USER_MODEL, null=True, on_delete=models.SET_NULL)),
                ('content_type', models.ForeignKey(to='contenttypes.ContentType', on_delete=models.CASCADE)),
            ],
            options={
            },
            bases=(models.Model,),
        ),
    ]



from django.conf import settings
from django_fsm_log.models import StateLog
import pytest


pytestmark = pytest.mark.ignore_article


def test_log_not_created_if_model_ignored(article):
    assert len(StateLog.objects.all()) == 0

    article.submit()
    article.save()

    assert len(StateLog.objects.all()) == 0


def test_log_created_on_transition_when_model_not_ignored(article):
    settings.DJANGO_FSM_LOG_IGNORED_MODELS = ['tests.models.SomeOtherModel']
    assert len(StateLog.objects.all()) == 0

    article.submit()
    article.save()

    assert len(StateLog.objects.all()) == 1



from django_fsm_log.models import StateLog
import pytest


def test_for_queryset_method_returns_only_logs_for_provided_object(article, article2):
    article2.submit()

    article.submit()
    article.publish()

    assert len(StateLog.objects.for_(article)) == 2
    for log in StateLog.objects.for_(article):
        assert article == log.content_object


@pytest.fixture
def create_kwargs(user, article):
    return {
        'by': user,
        'state': 'submitted',
        'transition': 'submit',
        'content_object': article
    }


@pytest.mark.pending_objects
def test_get_cache_key_for_object_returns_correctly_formatted_string(article):
    expected_result = 'StateLog:{}:{}'.format(
        article.__class__.__name__,
        article.pk
    )
    result = StateLog.pending_objects._get_cache_key_for_object(article)
    assert result == expected_result


@pytest.mark.pending_objects
def test_create_pending_sets_cache_item(article, create_kwargs, mocker):
    mock_cache = mocker.patch('django_fsm_log.managers.cache')
    expected_cache_key = StateLog.pending_objects._get_cache_key_for_object(article)
    StateLog.pending_objects.create(**create_kwargs)
    cache_key = mock_cache.set.call_args_list[0][0][0]
    cache_object = mock_cache.set.call_args_list[0][0][1]
    assert cache_key == expected_cache_key
    assert cache_object.state == create_kwargs['state']
    assert cache_object.transition == create_kwargs['transition']
    assert cache_object.content_object == create_kwargs['content_object']
    assert cache_object.by == create_kwargs['by']


@pytest.mark.pending_objects
def test_create_returns_correct_state_log(mocker, create_kwargs):
    mocker.patch('django_fsm_log.managers.cache')
    log = StateLog.pending_objects.create(**create_kwargs)
    assert log.state == create_kwargs['state']
    assert log.transition == create_kwargs['transition']
    assert log.content_object == create_kwargs['content_object']
    assert log.by == create_kwargs['by']


@pytest.mark.pending_objects
def test_commit_for_object_saves_log(mocker, article, create_kwargs):
    mock_cache = mocker.patch('django_fsm_log.managers.cache')
    log = StateLog.objects.create(**create_kwargs)
    mock_cache.get.return_value = log
    StateLog.pending_objects.commit_for_object(article)
    persisted_log = StateLog.objects.order_by('-pk').all()[0]
    assert log.state == persisted_log.state
    assert log.transition == persisted_log.transition
    assert log.content_object == persisted_log.content_object
    assert log.by == persisted_log.by


@pytest.mark.pending_objects
def test_commit_for_object_deletes_pending_log_from_cache(mocker, article, create_kwargs):
    mock_cache = mocker.patch('django_fsm_log.managers.cache')
    StateLog.pending_objects.create(**create_kwargs)
    StateLog.pending_objects.commit_for_object(article)
    mock_cache.delete.assert_called_once_with(StateLog.pending_objects._get_cache_key_for_object(article))


@pytest.mark.pending_objects
def test_get_for_object_calls_cache_get_with_correct_key(mocker, create_kwargs):
    mock_cache = mocker.patch('django_fsm_log.managers.cache')
    cache_key = StateLog.pending_objects._get_cache_key_for_object(create_kwargs['content_object'])
    StateLog.pending_objects.get_for_object(create_kwargs['content_object'])
    mock_cache.get.assert_called_once_with(cache_key)



# Generated by Django 2.1.3 on 2018-11-16 12:57

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('django_fsm_log', '0004_add_source_state'),
    ]

    operations = [
        migrations.AlterField(
            model_name='statelog',
            name='description',
            field=models.TextField(blank=True, null=True),
        ),
    ]



# Generated by Django 2.1.3 on 2018-11-16 12:57

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('django_fsm_log', '0005_description_null'),
    ]

    operations = [
        migrations.AlterField(
            model_name='statelog',
            name='object_id',
            field=models.TextField(db_index=True),
        ),
    ]



import ast
import inspect

import django
from django.core.checks import (
    Warning,
    register,
    Tags,
)
from django.core.exceptions import FieldDoesNotExist


@register(Tags.compatibility)
def integer_object_id_check(app_configs, **kwargs):
    errors = []
    for app in django.apps.apps.get_app_configs():

        # Skip third party apps.
        if app.path.find('site-packages') > -1:
            continue

        for model in app.get_models():
            for check_message in check_model_for_integer_object_id(model):
                errors.append(check_message)
    return errors


def check_model_for_integer_object_id(model):
    """Check a single model.

    Yields (django.checks.CheckMessage)
    """

    from django_fsm_log.models import StateLog

    model_source = inspect.getsource(model)
    model_node = ast.parse(model_source)

    for node in model_node.body[0].body:

        # Check if node is a model field.
        if not isinstance(node, ast.Assign):
            continue

        if len(node.targets) != 1:
            continue

        if not isinstance(node.targets[0], ast.Name):
            continue

        field_name = node.targets[0].id
        try:
            field = model._meta.get_field(field_name)
        except FieldDoesNotExist:
            continue

        # Node is a model field here

        # Check if field has foreign key to StateLog defined
        for kw in node.value.keywords:
            if kw.arg == 'to':
                to = kw
                if to == 'django_fsm_log.StateLog' or isinstance(to, StateLog):
                    yield Warning(
                        'StateLog has changed its object_id from PositiveIntegerField to TextField to better support '
                        'the primary key types in Django.',
                        hint='Migration PositiveIntegerField -> TextField is potentially needed on field {}.'.format(
                            field.name
                        ),
                        obj=field,
                        id='django_fsm_log.W001',
                    )



from django_fsm import TransitionNotAllowed
from django_fsm_log.models import StateLog
import pytest

from .models import Article, ArticleInteger


def test_get_available_state_transitions(article):
    assert len(list(article.get_available_state_transitions())) == 4


def test_get_all_state_transitions(article):
    assert len(list(article.get_all_state_transitions())) == 6


def test_log_created_on_transition(article):
    assert len(StateLog.objects.all()) == 0

    article.submit()
    article.save()

    assert len(StateLog.objects.all()) == 1


def test_log_not_created_if_transition_fails(article):
    assert len(StateLog.objects.all()) == 0

    with pytest.raises(TransitionNotAllowed):
        article.publish()

    assert len(StateLog.objects.all()) == 0


def test_log_not_created_if_target_is_none(article):
    assert len(StateLog.objects.all()) == 0

    article.validate_draft()

    assert len(StateLog.objects.all()) == 0


def test_by_is_set_when_passed_into_transition(article, user):
    article.submit(by=user)

    log = StateLog.objects.all()[0]
    assert user == log.by
    with pytest.raises(AttributeError):
        getattr(article, '__django_fsm_log_attr_by')


def test_by_is_none_when_not_set_in_transition(article):
    article.submit()

    log = StateLog.objects.all()[0]
    assert log.by is None


def test_description_is_set_when_passed_into_transition(article):
    description = "Lorem ipsum"
    article.submit(description=description)

    log = StateLog.objects.all()[0]
    assert description == log.description
    with pytest.raises(AttributeError):
        getattr(article, '__django_fsm_log_attr_description')


def test_description_is_none_when_not_set_in_transition(article):
    article.submit()

    log = StateLog.objects.all()[0]
    assert log.description is None


def test_description_can_be_mutated_by_the_transition(article):
    description = "Sed egestas dui"
    article.submit_inline_description_change(change_to=description)

    log = StateLog.objects.all()[0]
    assert description == log.description
    with pytest.raises(AttributeError):
        getattr(article, '__django_fsm_log_attr_description')


def test_logged_state_is_new_state(article):
    article.submit()

    log = StateLog.objects.all()[0]
    assert log.state == 'submitted'


def test_logged_transition_is_name_of_transition_method(article):
    article.submit()

    log = StateLog.objects.all()[0]
    assert log.transition == 'submit'


def test_logged_content_object_is_instance_being_transitioned(article):
    article.submit()

    log = StateLog.objects.all()[0]
    assert log.content_object == article


def test_get_display_state(article):
    article.submit()
    article.save()

    log = StateLog.objects.latest()
    article = Article.objects.get(pk=article.pk)
    prev_state = article.get_state_display()

    assert log.get_state_display() == prev_state

    article.publish()
    article.save()

    log = StateLog.objects.latest()

    article = Article.objects.get(pk=article.pk)

    assert log.get_state_display() == article.get_state_display()
    assert log.get_source_state_display() == prev_state


def test_get_display_state_with_integer(article_integer):
    article_integer.change_to_two()
    article_integer.save()

    log = StateLog.objects.latest()
    article_integer = ArticleInteger.objects.get(pk=article_integer.pk)

    assert log.get_state_display() == article_integer.get_state_display()



import django
from django.db import models
from django.db.models.query import QuerySet
from django.contrib.contenttypes.models import ContentType
from django_fsm_log.backends import cache


class StateLogQuerySet(QuerySet):
    def _get_content_type(self, obj):
        return ContentType.objects.get_for_model(obj)

    def for_(self, obj):
        return self.filter(
            content_type=self._get_content_type(obj),
            object_id=obj.pk
        )


class StateLogManager(models.Manager):
    def get_queryset(self):
        return StateLogQuerySet(self.model)

    if django.VERSION < (1, 7):
        get_query_set = get_queryset

    def __getattr__(self, attr, *args):
        # see https://code.djangoproject.com/ticket/15062 for details
        if attr.startswith("_"):
            raise AttributeError
        return getattr(self.get_queryset(), attr, *args)


class PendingStateLogManager(models.Manager):
    def _get_cache_key_for_object(self, obj):
        return 'StateLog:{}:{}'.format(
            obj.__class__.__name__,
            obj.pk
        )

    def create(self, *args, **kwargs):
        log = self.model(**kwargs)
        key = self._get_cache_key_for_object(kwargs['content_object'])
        cache.set(key, log, 10)
        return log

    def commit_for_object(self, obj):
        key = self._get_cache_key_for_object(obj)
        log = self.get_for_object(obj)
        log.save()
        cache.delete(key)
        return log

    def get_for_object(self, obj):
        key = self._get_cache_key_for_object(obj)
        return cache.get(key)



try:
    from django.urls import reverse
except ImportError:
    # django<=1.10
    from django.core.urlresolvers import reverse


def test_state_log_inline_django2(article, admin_client, admin_user):
    article.submit(by=admin_user)
    article.publish(by=admin_user)
    url = reverse('admin:tests_article_change', args=(article.pk,))
    response = admin_client.get(url)
    assert response.status_code == 200
    assert '{} - submit'.format(article).encode() in response.content
    assert '{} - publish'.format(article).encode() in response.content



import os

BASE_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': os.path.join(BASE_DIR, 'db.sqlite3'),
    }
}
INSTALLED_APPS = [
    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django_fsm_log',
    'tests',
]
MIDDLEWARE = MIDDLEWARE_CLASSES = [
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
]
SECRET_KEY = 'abc123'
ROOT_URLCONF = 'tests.urls'
TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'APP_DIRS': True,
        'OPTIONS': {
            'context_processors': [
                'django.contrib.auth.context_processors.auth',
            ],
        },
    },
]



from django_fsm_log.conf import settings
from .helpers import FSMLogDescriptor


def _pre_transition_callback(sender, instance, name, source, target, manager, **kwargs):

    if BaseBackend._get_model_qualified_name__(sender) in settings.DJANGO_FSM_LOG_IGNORED_MODELS:
        return

    if target is None:
        return

    values = {
        'source_state': source,
        'state': target,
        'transition': name,
        'content_object': instance,
    }
    try:
        values['by'] = FSMLogDescriptor(instance, 'by').get()
    except AttributeError:
        pass
    try:
        values['description'] = FSMLogDescriptor(instance, 'description').get()
    except AttributeError:
        pass

    manager.create(**values)


class BaseBackend(object):

    @staticmethod
    def setup_model(model):
        raise NotImplementedError

    @staticmethod
    def pre_transition_callback(*args, **kwargs):
        raise NotImplementedError

    @staticmethod
    def post_transition_callback(*args, **kwargs):
        raise NotImplementedError

    @staticmethod
    def _get_model_qualified_name__(sender):
        return '%s.%s' % (sender.__module__,
                          getattr(sender, '__qualname__', sender.__name__))


class CachedBackend(BaseBackend):

    @staticmethod
    def setup_model(model):
        from .managers import PendingStateLogManager
        model.add_to_class('pending_objects', PendingStateLogManager())

    @staticmethod
    def pre_transition_callback(sender, instance, name, source, target, **kwargs):
        from .models import StateLog
        return _pre_transition_callback(sender, instance, name, source, target, StateLog.pending_objects, **kwargs)

    @staticmethod
    def post_transition_callback(sender, instance, name, source, target, **kwargs):
        from .models import StateLog
        StateLog.pending_objects.commit_for_object(instance)


class SimpleBackend(BaseBackend):

    @staticmethod
    def setup_model(model):
        pass

    @staticmethod
    def pre_transition_callback(sender, **kwargs):
        pass

    @staticmethod
    def post_transition_callback(sender, instance, name, source, target, **kwargs):
        from .models import StateLog
        return _pre_transition_callback(sender, instance, name, source, target, StateLog.objects, **kwargs)


if settings.DJANGO_FSM_LOG_STORAGE_METHOD == 'django_fsm_log.backends.CachedBackend':
    try:
        from django.core.cache import caches
    except ImportError:
        from django.core.cache import get_cache  # Deprecated, removed in 1.9.
        cache = get_cache(settings.DJANGO_FSM_LOG_CACHE_BACKEND)
    else:
        cache = caches[settings.DJANGO_FSM_LOG_CACHE_BACKEND]
else:
    cache = None



from django.conf import settings  # noqa
from appconf import AppConf


class DjangoFSMLogConf(AppConf):
    STORAGE_METHOD = 'django_fsm_log.backends.SimpleBackend'
    CACHE_BACKEND = 'default'
    IGNORED_MODELS = []



# Generated by Django 2.1.4 on 2018-12-13 10:54

from django.db import migrations, models


class Migration(migrations.Migration):

    replaces = [('django_fsm_log', '0003_statelog_description'), ('django_fsm_log', '0004_add_source_state'), ('django_fsm_log', '0005_description_null')]

    dependencies = [
        ('django_fsm_log', '0002_auto_20151207_1521'),
    ]

    operations = [
        migrations.AddField(
            model_name='statelog',
            name='description',
            field=models.TextField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name='statelog',
            name='source_state',
            field=models.CharField(blank=True, db_index=True, default=None, max_length=255, null=True),
        ),
        migrations.AlterField(
            model_name='statelog',
            name='state',
            field=models.CharField(db_index=True, max_length=255, verbose_name='Target state'),
        ),
    ]



from django.contrib.contenttypes.admin import GenericTabularInline
from django.db.models import F

from .models import StateLog


__all__ = ('StateLogInline',)


class StateLogInline(GenericTabularInline):
    model = StateLog
    can_delete = False

    def has_add_permission(self, request, obj=None):
        return False

    def has_change_permission(self, request, obj=None):
        return True

    fields = (
        'transition',
        'source_state',
        'state',
        'by',
        'description',
        'timestamp',
    )

    def get_readonly_fields(self, request, obj=None):
        return self.fields

    def get_queryset(self, request):
        return super(StateLogInline, self).get_queryset(
            request
        ).order_by(F('timestamp').desc())



# -*- coding:utf-8 -*-
from __future__ import unicode_literals

from django.contrib.contenttypes.fields import GenericForeignKey
from django.contrib.contenttypes.models import ContentType
from django.db import models
from django.utils.timezone import now
from django_fsm import FSMFieldMixin, FSMIntegerField

from .conf import settings
from .managers import StateLogManager


class StateLog(models.Model):
    timestamp = models.DateTimeField(default=now)
    by = models.ForeignKey(getattr(settings, 'AUTH_USER_MODEL', 'auth.User'), blank=True,
                           null=True, on_delete=models.SET_NULL)
    source_state = models.CharField(max_length=255, db_index=True, null=True, blank=True,
                                    default=None)
    state = models.CharField("Target state", max_length=255, db_index=True)
    transition = models.CharField(max_length=255)

    content_type = models.ForeignKey(ContentType, on_delete=models.CASCADE)
    object_id = models.TextField(db_index=True)
    content_object = GenericForeignKey('content_type', 'object_id')

    description = models.TextField(blank=True, null=True)

    objects = StateLogManager()

    class Meta:
        get_latest_by = 'timestamp'

    def __str__(self):
        return '{} - {} - {}'.format(
            self.timestamp,
            self.content_object,
            self.transition
        )

    def get_state_display(self, field_name='state'):
        fsm_cls = self.content_type.model_class()
        for field in fsm_cls._meta.fields:
            state = getattr(self, field_name)
            if isinstance(field, FSMIntegerField):
                state_display = dict(field.flatchoices).get(int(state), state)
                return str(state_display)
            elif isinstance(field, FSMFieldMixin):
                state_display = dict(field.flatchoices).get(state, state)
                return str(state_display)

    def get_source_state_display(self):
        return self.get_state_display('source_state')



# -*- coding: utf-8 -*-
# Generated by Django 1.9 on 2015-12-07 15:21
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('django_fsm_log', '0001_initial'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='statelog',
            options={'get_latest_by': 'timestamp'},
        ),
    ]



from django.contrib import admin
from django_fsm_log.admin import StateLogInline

from .models import Article


class ArticleAdmin(admin.ModelAdmin):
    inlines = [StateLogInline]


admin.site.register(Article, ArticleAdmin)



try:
    from django.urls import path
    from django.contrib import admin

    urlpatterns = [path('admin', admin.site.urls)]
except ImportError:
    # django < 2.0
    from django.conf.urls import include, url
    from django.contrib import admin

    urlpatterns = [url(r'^admin/', include(admin.site.urls))]



from functools import wraps, partial
from .helpers import FSMLogDescriptor


def fsm_log_by(func):
    @wraps(func)
    def wrapped(instance, *args, **kwargs):
        try:
            by = kwargs['by']
        except KeyError:
            return func(instance, *args, **kwargs)
        with FSMLogDescriptor(instance, 'by', by):
            return func(instance, *args, **kwargs)

    return wrapped


def fsm_log_description(func=None, allow_inline=False):
    if func is None:
        return partial(fsm_log_description, allow_inline=allow_inline)

    @wraps(func)
    def wrapped(instance, *args, **kwargs):
        with FSMLogDescriptor(instance, 'description') as descriptor:
            try:
                description = kwargs['description']
            except KeyError:
                if allow_inline:
                    kwargs['description'] = descriptor
                return func(instance, *args, **kwargs)
            descriptor.set(description)
            return func(instance, *args, **kwargs)

    return wrapped



#!/usr/bin/env python

from setuptools import setup, find_packages


def readfile(filename):
    with open(filename, 'r') as open_file:
        return open_file.read()


setup(
    name='django-fsm-logger',
    version='2.0.2',
    description='Logging for django-fsm',
    long_description=readfile('README.md'),
    long_description_content_type='text/markdown',
    author='Gizmag',
    author_email='tech@gizmag.com',
    url='https://github.com/diegoponciano/django-fsm-logger',
    license='MIT',
    packages=find_packages(),
    install_requires=['django>=1.8', 'django_fsm>=2', 'django_appconf'],
    extras_require={
        'testing': [
            'pytest',
            'pytest-cov',
            'pytest-django',
            'pytest-mock',
        ],
    },
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Framework :: Django :: 1.11',
        'Framework :: Django :: 2.0',
        'Framework :: Django :: 2.1',
        'Framework :: Django :: 2.2',
        'Framework :: Django :: 3.0',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ]
)



# -*- coding: utf-8 -*-
# Generated by Django 1.11 on 2017-12-22 00:19
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('django_fsm_log', '0002_auto_20151207_1521'),
    ]

    operations = [
        migrations.AddField(
            model_name='statelog',
            name='description',
            field=models.TextField(blank=True),
        ),
    ]



from __future__ import unicode_literals

from django.db import models
from django_fsm_log.decorators import fsm_log_by, fsm_log_description

from django_fsm import FSMField, FSMIntegerField, transition


class Article(models.Model):
    STATES = (
        ('draft', 'Draft'),
        ('submitted', 'Article submitted'),
        ('published', 'Article published'),
        ('deleted', 'Article deleted'),
    )

    state = FSMField(choices=STATES, default='draft', protected=True)

    @fsm_log_by
    @fsm_log_description
    @transition(field=state, source='draft', target='submitted')
    def submit(self, description=None, by=None):
        pass

    @fsm_log_by
    @transition(field=state, source='submitted', target='draft')
    def request_changes(self, by=None):
        pass

    @fsm_log_by
    @transition(field=state, source='submitted', target='published')
    def publish(self, by=None):
        pass

    @fsm_log_by
    @transition(field=state, source='*', target='deleted')
    def delete(self, using=None):
        pass

    @fsm_log_by
    @fsm_log_description(allow_inline=True)
    @transition(field=state, source='draft', target='submitted')
    def submit_inline_description_change(self, change_to, description=None, by=None):
        description.set(change_to)

    @fsm_log_by
    @transition(field=state, source='draft', target=None)
    def validate_draft(self, by=None):
        pass


class ArticleInteger(models.Model):
    STATE_ONE = 1
    STATE_TWO = 2

    STATES = (
        (STATE_ONE, 'one'),
        (STATE_TWO, 'two'),
    )

    state = FSMIntegerField(choices=STATES, default=STATE_ONE)

    @fsm_log_by
    @transition(field=state, source=STATE_ONE, target=STATE_TWO)
    def change_to_two(self, by=None):
        pass
